-------------------------------------------------------------------------------
--
-- Copyright (C) 2013
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, unb1_board_lib, mm_lib, remu_lib, epcs_lib, eth_lib, technology_lib, tech_tse_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.tb_common_mem_pkg.ALL;
USE common_lib.common_field_pkg.ALL;
USE common_lib.common_network_total_header_pkg.ALL;
USE common_lib.common_network_layers_pkg.ALL;
USE unb1_board_lib.unb1_board_pkg.ALL;
USE unb1_board_lib.unb1_board_peripherals_pkg.ALL;
USE mm_lib.mm_file_pkg.ALL;
USE mm_lib.mm_file_unb_pkg.ALL;
USE eth_lib.eth_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE tech_tse_lib.tech_tse_pkg.ALL;
USE tech_tse_lib.tb_tech_tse_pkg.ALL;
USE work.qsys_unb1_test_pkg.ALL;


ENTITY mmm_unb1_test IS
  GENERIC (
    g_sim           : BOOLEAN := FALSE; --FALSE: use QSYS; TRUE: use mm_file I/O
    g_sim_unb_nr    : NATURAL := 0;
    g_sim_node_nr   : NATURAL := 0;
    g_nof_streams   : NATURAL;
    g_bg_block_size : NATURAL;
    g_hdr_field_arr : t_common_field_arr
  );
  PORT (
    xo_clk                   : IN  STD_LOGIC;
    xo_rst_n                 : IN  STD_LOGIC;
    xo_rst                   : IN  STD_LOGIC;

    mm_rst                   : IN  STD_LOGIC;
    mm_clk                   : OUT STD_LOGIC;
    mm_locked                : OUT STD_LOGIC;

    epcs_clk                 : OUT STD_LOGIC;
    cal_rec_clk              : OUT STD_LOGIC;
    dp_clk                   : OUT STD_LOGIC;


    pout_wdi                 : OUT STD_LOGIC;
                             
    -- Manual WDI override
    reg_wdi_mosi             : OUT t_mem_mosi;
    reg_wdi_miso             : IN  t_mem_miso;
                             
    -- system_info
    reg_unb_system_info_mosi : OUT t_mem_mosi;
    reg_unb_system_info_miso : IN  t_mem_miso;
    rom_unb_system_info_mosi : OUT t_mem_mosi;
    rom_unb_system_info_miso : IN  t_mem_miso;
                             
    -- UniBoard I2C sensors
    reg_unb_sens_mosi        : OUT t_mem_mosi; 
    reg_unb_sens_miso        : IN  t_mem_miso; 
                             
    -- PPSH
    reg_ppsh_mosi            : OUT t_mem_mosi; 
    reg_ppsh_miso            : IN  t_mem_miso; 
                             
    -- eth1g
    eth1g_tse_clk            : OUT STD_LOGIC;
    eth1g_mm_rst             : OUT STD_LOGIC;
    eth1g_tse_mosi           : OUT t_mem_mosi;  
    eth1g_tse_miso           : IN  t_mem_miso;  
    eth1g_reg_mosi           : OUT t_mem_mosi;  
    eth1g_reg_miso           : IN  t_mem_miso;  
    eth1g_reg_interrupt      : IN  STD_LOGIC; 
    eth1g_ram_mosi           : OUT t_mem_mosi;  
    eth1g_ram_miso           : IN  t_mem_miso;

    -- EPCS read
    reg_dpmm_data_mosi       : OUT t_mem_mosi;
    reg_dpmm_data_miso       : IN  t_mem_miso;
    reg_dpmm_ctrl_mosi       : OUT t_mem_mosi;
    reg_dpmm_ctrl_miso       : IN  t_mem_miso;

    -- EPCS write
    reg_mmdp_data_mosi       : OUT t_mem_mosi;
    reg_mmdp_data_miso       : IN  t_mem_miso;
    reg_mmdp_ctrl_mosi       : OUT t_mem_mosi;
    reg_mmdp_ctrl_miso       : IN  t_mem_miso;

    -- EPCS status/control
    reg_epcs_mosi            : OUT t_mem_mosi;
    reg_epcs_miso            : IN  t_mem_miso;

    -- Remote Update
    reg_remu_mosi            : OUT t_mem_mosi;
    reg_remu_miso            : IN  t_mem_miso;

    ram_diag_bg_mosi               : OUT t_mem_mosi;
    ram_diag_bg_miso               : IN  t_mem_miso;
    reg_diag_bg_mosi               : OUT t_mem_mosi;
    reg_diag_bg_miso               : IN  t_mem_miso;

    reg_dp_offload_tx_mosi         : OUT t_mem_mosi;
    reg_dp_offload_tx_miso         : IN  t_mem_miso;

    reg_dp_offload_tx_hdr_dat_mosi : OUT t_mem_mosi;
    reg_dp_offload_tx_hdr_dat_miso : IN  t_mem_miso;

    reg_dp_offload_tx_hdr_ovr_mosi : OUT t_mem_mosi;
    reg_dp_offload_tx_hdr_ovr_miso : IN  t_mem_miso;

    reg_dp_offload_rx_hdr_dat_mosi : OUT t_mem_mosi;
    reg_dp_offload_rx_hdr_dat_miso : IN  t_mem_miso;

    reg_bsn_monitor_mosi           : OUT t_mem_mosi;
    reg_bsn_monitor_miso           : IN  t_mem_miso;

    ram_diag_data_buf_mosi         : OUT t_mem_mosi;
    ram_diag_data_buf_miso         : IN  t_mem_miso;
    reg_diag_data_buf_mosi         : OUT t_mem_mosi;
    reg_diag_data_buf_miso         : IN  t_mem_miso;

    reg_tr_10GbE_mosi        : OUT t_mem_mosi;
    reg_tr_10GbE_miso        : IN  t_mem_miso;
    reg_tr_xaui_mosi         : OUT t_mem_mosi;
    reg_tr_xaui_miso         : IN  t_mem_miso
  );
END mmm_unb1_test;

ARCHITECTURE str OF mmm_unb1_test IS

  SIGNAL i_mm_clk        : STD_LOGIC := '1';
  SIGNAL i_cal_rec_clk   : STD_LOGIC := '1';
  SIGNAL i_dp_clk        : STD_LOGIC := '1';
  SIGNAL i_eth1g_tse_clk : STD_LOGIC := '1';
  SIGNAL i_epcs_clk      : STD_LOGIC := '1';

  -- Block generator
  CONSTANT c_ram_diag_bg_addr_w                    : NATURAL := ceil_log2(g_nof_streams* pow2(ceil_log2(g_bg_block_size)));

  -- dp_offload
  CONSTANT c_reg_dp_offload_tx_adr_w               : NATURAL := 1; -- Dev note: add to c_unb1_board_peripherals_mm_reg_default
  CONSTANT c_reg_dp_offload_tx_multi_adr_w         : NATURAL := ceil_log2(g_nof_streams* pow2(c_reg_dp_offload_tx_adr_w));

  CONSTANT c_reg_dp_offload_tx_hdr_dat_nof_words   : NATURAL := field_nof_words(g_hdr_field_arr, c_word_w);
  CONSTANT c_reg_dp_offload_tx_hdr_dat_adr_w       : NATURAL := ceil_log2(c_reg_dp_offload_tx_hdr_dat_nof_words);
  CONSTANT c_reg_dp_offload_tx_hdr_dat_multi_adr_w : NATURAL := ceil_log2(g_nof_streams* pow2(c_reg_dp_offload_tx_hdr_dat_adr_w));

  CONSTANT c_reg_dp_offload_tx_hdr_ovr_nof_words   : NATURAL := g_hdr_field_arr'LENGTH;
  CONSTANT c_reg_dp_offload_tx_hdr_ovr_adr_w       : NATURAL := ceil_log2(c_reg_dp_offload_tx_hdr_ovr_nof_words);
  CONSTANT c_reg_dp_offload_tx_hdr_ovr_multi_adr_w : NATURAL := ceil_log2(g_nof_streams* pow2(c_reg_dp_offload_tx_hdr_ovr_adr_w));

  CONSTANT c_reg_dp_offload_rx_hdr_dat_nof_words   : NATURAL := field_nof_words(g_hdr_field_arr, c_word_w);
  CONSTANT c_reg_dp_offload_rx_hdr_dat_adr_w       : NATURAL := ceil_log2(c_reg_dp_offload_rx_hdr_dat_nof_words);
  CONSTANT c_reg_dp_offload_rx_hdr_dat_multi_adr_w : NATURAL := ceil_log2(g_nof_streams* pow2(c_reg_dp_offload_rx_hdr_dat_adr_w));

  -- tr_10GbE
  CONSTANT c_reg_tr_10GbE_adr_w                    : NATURAL := 13;
  CONSTANT c_reg_tr_10GbE_multi_adr_w              : NATURAL := ceil_log2(g_nof_streams* pow2(c_reg_tr_10GbE_adr_w));

  -- tr_xaui
  CONSTANT c_reg_tr_xaui_adr_w                     : NATURAL := 9;
  CONSTANT c_reg_tr_xaui_multi_adr_w               : NATURAL := ceil_log2(g_nof_streams* pow2(c_reg_tr_xaui_adr_w));

  -- BSN monitors
  CONSTANT c_reg_rsp_bsn_monitor_adr_w             : NATURAL := ceil_log2(g_nof_streams* pow2(c_unb1_board_peripherals_mm_reg_default.reg_bsn_monitor_adr_w));


  CONSTANT c_dp_reg_mm_nof_words       : NATURAL := 1;
  CONSTANT c_dp_reg_mm_adr_w           : NATURAL := ceil_log2(c_eth_nof_udp_ports * pow2(ceil_log2(c_dp_reg_mm_nof_words)));

  -- Simulation
  CONSTANT c_mm_clk_period      : TIME := 8 ns;   -- 125 MHz
  CONSTANT c_cal_rec_clk_period : TIME := 25 ns;  -- 40 MHz
  CONSTANT c_dp_clk_period      : TIME := 5 ns;   -- 200 MHz
  CONSTANT c_tse_clk_period     : TIME := 8 ns;   -- 125 MHz
  CONSTANT c_epcs_clk_period    : TIME := 50 ns;  -- 20 MHz

  CONSTANT c_sim_node_type : STRING(1 TO 2):= sel_a_b(g_sim_node_nr<4, "FN", "BN");
  CONSTANT c_sim_node_nr   : NATURAL := sel_a_b(c_sim_node_type="BN", g_sim_node_nr-4, g_sim_node_nr);



  CONSTANT c_sim_eth_src_mac                       : STD_LOGIC_VECTOR(c_network_eth_mac_slv'RANGE) := X"00228608" & TO_UVEC(g_sim_unb_nr, c_byte_w) & TO_UVEC(g_sim_node_nr, c_byte_w);
  CONSTANT c_sim_eth_control_rx_en                 : NATURAL := 2**c_eth_mm_reg_control_bi.rx_en;

  SIGNAL sim_eth_mm_bus_switch                     : STD_LOGIC;
  SIGNAL sim_eth_psc_access                        : STD_LOGIC;

  SIGNAL i_eth1g_reg_mosi                          : t_mem_mosi;
  SIGNAL i_eth1g_reg_miso                          : t_mem_miso;

  SIGNAL sim_eth1g_reg_mosi                        : t_mem_mosi;


  
  ----------------------------------------------------------------------------
  -- mm_file component
  ----------------------------------------------------------------------------
  COMPONENT mm_file
  GENERIC(
    g_file_prefix       : STRING;
    g_mm_clk_period     : TIME := c_mm_clk_period;
    g_update_on_change  : BOOLEAN := FALSE;
    g_mm_rd_latency     : NATURAL := 1
  );
  PORT (
    mm_rst        : IN  STD_LOGIC;
    mm_clk        : IN  STD_LOGIC;
    mm_master_out : OUT t_mem_mosi;
    mm_master_in  : IN  t_mem_miso 
  );
  END COMPONENT;

BEGIN

  mm_clk        <= i_mm_clk;
  cal_rec_clk   <= i_cal_rec_clk;
  dp_clk        <= i_dp_clk;
  eth1g_tse_clk <= i_eth1g_tse_clk;
  epcs_clk      <= i_epcs_clk;

  ----------------------------------------------------------------------------
  -- MM <-> file I/O for simulation. The files are created in $UPE/sim.
  ----------------------------------------------------------------------------
  gen_mm_file_io : IF g_sim = TRUE GENERATE

    i_mm_clk         <= NOT i_mm_clk        AFTER c_mm_clk_period/2;
    i_cal_rec_clk    <= NOT i_cal_rec_clk   AFTER c_cal_rec_clk_period/2;
    i_dp_clk         <= NOT i_dp_clk        AFTER c_dp_clk_period/2;
    i_eth1g_tse_clk  <= NOT i_eth1g_tse_clk AFTER c_tse_clk_period/2;
    i_epcs_clk       <= NOT i_epcs_clk      AFTER c_epcs_clk_period/2;

    mm_locked        <= '0', '1'            AFTER c_mm_clk_period*5;
    eth1g_mm_rst     <= '1', '0'            AFTER c_tse_clk_period*5;


    u_mm_file_reg_unb_system_info : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "PIO_SYSTEM_INFO")
                                               PORT MAP(mm_rst, i_mm_clk, reg_unb_system_info_mosi, reg_unb_system_info_miso );

    u_mm_file_rom_unb_system_info : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "ROM_SYSTEM_INFO")
                                               PORT MAP(mm_rst, i_mm_clk, rom_unb_system_info_mosi, rom_unb_system_info_miso );

    u_mm_file_reg_wdi             : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_WDI")
                                               PORT MAP(mm_rst, i_mm_clk, reg_wdi_mosi, reg_wdi_miso );

    u_mm_file_reg_unb_sens        : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_UNB_SENS")
                                               PORT MAP(mm_rst, i_mm_clk, reg_unb_sens_mosi, reg_unb_sens_miso );

    u_mm_file_reg_ppsh            : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "PIO_PPS")
                                               PORT MAP(mm_rst, i_mm_clk, reg_ppsh_mosi, reg_ppsh_miso );

    u_mm_file_reg_diag_bg               : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DIAG_BG")
                                                     PORT MAP(mm_rst, i_mm_clk, reg_diag_bg_mosi, reg_diag_bg_miso);

    u_mm_file_ram_diag_bg               : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "RAM_DIAG_BG")
                                                     PORT MAP(mm_rst, i_mm_clk, ram_diag_bg_mosi, ram_diag_bg_miso);

    u_mm_file_reg_dp_offload_tx         : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DP_OFFLOAD_TX")
                                                     PORT MAP(mm_rst, i_mm_clk, reg_dp_offload_tx_mosi, reg_dp_offload_tx_miso );

    u_mm_file_reg_dp_offload_tx_hdr_dat : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DP_OFFLOAD_TX_HDR_DAT")
                                                     PORT MAP(mm_rst, i_mm_clk, reg_dp_offload_tx_hdr_dat_mosi, reg_dp_offload_tx_hdr_dat_miso );

    u_mm_file_reg_dp_offload_tx_hdr_ovr : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DP_OFFLOAD_TX_HDR_OVR")
                                                     PORT MAP(mm_rst, i_mm_clk, reg_dp_offload_tx_hdr_ovr_mosi, reg_dp_offload_tx_hdr_ovr_miso );

    u_mm_file_reg_dp_offload_rx_hdr_dat : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DP_OFFLOAD_RX_HDR_DAT")
                                                     PORT MAP(mm_rst, i_mm_clk, reg_dp_offload_rx_hdr_dat_mosi, reg_dp_offload_rx_hdr_dat_miso );

    u_mm_file_reg_bsn_monitor           : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_BSN_MONITOR")
                                                     PORT MAP(mm_rst, i_mm_clk, reg_bsn_monitor_mosi, reg_bsn_monitor_miso );

    u_mm_file_ram_diag_data_buffer      : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "RAM_DIAG_DATA_BUFFER")
                                                     PORT MAP(mm_rst, i_mm_clk, ram_diag_data_buf_mosi, ram_diag_data_buf_miso);

    u_mm_file_reg_diag_data_buffer      : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_DIAG_DATA_BUFFER")
                                                     PORT MAP(mm_rst, i_mm_clk, reg_diag_data_buf_mosi, reg_diag_data_buf_miso);

    -- Note: the eth1g RAM and TSE buses are only required by unb_osy on the NIOS as they provide the ethernet<->MM gateway.
    u_mm_file_reg_eth             : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "AVS_ETH_0_MMS_REG")
                                               PORT MAP(mm_rst, i_mm_clk, eth1g_reg_mosi, eth1g_reg_miso );

    u_mm_file_reg_tr_10GbE        : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_TR_10GBE", c_mm_clk_period, FALSE, 0)
                                               PORT MAP(mm_rst, i_mm_clk, reg_tr_10GbE_mosi, reg_tr_10GbE_miso );

    u_mm_file_reg_tr_xaui         : mm_file GENERIC MAP(mmf_unb_file_prefix(g_sim_unb_nr, c_sim_node_nr, c_sim_node_type) & "REG_TR_XAUI", c_mm_clk_period, FALSE, 0)
                                               PORT MAP(mm_rst, i_mm_clk, reg_tr_xaui_mosi, reg_tr_xaui_miso );


    ----------------------------------------------------------------------------
    -- 1GbE setup sequence normally performed by unb_os@NIOS
    ----------------------------------------------------------------------------
    p_eth_setup : PROCESS
    BEGIN
      sim_eth_mm_bus_switch <= '1';

      eth1g_tse_mosi.wr <= '0';
      eth1g_tse_mosi.rd <= '0';
      WAIT FOR 400 ns;
      WAIT UNTIL rising_edge(i_mm_clk);
      proc_tech_tse_setup(c_tech_stratixiv, FALSE, c_tech_tse_tx_fifo_depth, c_tech_tse_rx_fifo_depth, c_tech_tse_tx_ready_latency, c_sim_eth_src_mac, sim_eth_psc_access, i_mm_clk, eth1g_tse_miso, eth1g_tse_mosi);

      -- Enable RX
      proc_mem_mm_bus_wr(c_eth_reg_control_wi+0, c_sim_eth_control_rx_en, i_mm_clk, eth1g_reg_miso, sim_eth1g_reg_mosi);  -- control rx en
      sim_eth_mm_bus_switch <= '0';

      WAIT;
    END PROCESS;

    p_switch : PROCESS(sim_eth_mm_bus_switch, sim_eth1g_reg_mosi, i_eth1g_reg_mosi)
    BEGIN
      IF sim_eth_mm_bus_switch = '1' THEN
          eth1g_reg_mosi <= sim_eth1g_reg_mosi;
        ELSE
          eth1g_reg_mosi <= i_eth1g_reg_mosi;
        END IF;
    END PROCESS;


    ----------------------------------------------------------------------------
    -- Procedure that polls a sim control file that can be used to e.g. get
    -- the simulation time in ns
    ----------------------------------------------------------------------------
    mmf_poll_sim_ctrl_file(c_mmf_unb_file_path & "sim.ctrl", c_mmf_unb_file_path & "sim.stat");

  END GENERATE;


  ----------------------------------------------------------------------------
  -- QSYS 
  ----------------------------------------------------------------------------
  gen_qsys : IF g_sim = FALSE GENERATE
    u_qsys : qsys_unb1_test
    PORT MAP (
      clk_0                                         => xo_clk, 
      reset_n                                       => xo_rst_n,
      mm_clk                                        => i_mm_clk,  
      cal_reconf_clk                                => i_cal_rec_clk,
      tse_clk                                       => i_eth1g_tse_clk,
      epcs_clk                                      => i_epcs_clk,
      dp_clk                                        => i_dp_clk,
  
       -- the_altpll_0
      locked_from_the_altpll_0                      => mm_locked,
      phasedone_from_the_altpll_0                   => OPEN,
      areset_to_the_altpll_0                        => xo_rst,
  
      -- the_avs_eth_0
      coe_clk_export_from_the_avs_eth_0             => OPEN,
      coe_reset_export_from_the_avs_eth_0           => eth1g_mm_rst,
      coe_tse_address_export_from_the_avs_eth_0     => eth1g_tse_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_tse_adr_w-1 DOWNTO 0),
      coe_tse_write_export_from_the_avs_eth_0       => eth1g_tse_mosi.wr,
      coe_tse_writedata_export_from_the_avs_eth_0   => eth1g_tse_mosi.wrdata(c_word_w-1 DOWNTO 0),
      coe_tse_read_export_from_the_avs_eth_0        => eth1g_tse_mosi.rd,
      coe_tse_readdata_export_to_the_avs_eth_0      => eth1g_tse_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_tse_waitrequest_export_to_the_avs_eth_0   => eth1g_tse_miso.waitrequest,
      coe_reg_address_export_from_the_avs_eth_0     => eth1g_reg_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_eth_adr_w-1 DOWNTO 0),
      coe_reg_write_export_from_the_avs_eth_0       => eth1g_reg_mosi.wr,
      coe_reg_writedata_export_from_the_avs_eth_0   => eth1g_reg_mosi.wrdata(c_word_w-1 DOWNTO 0),
      coe_reg_read_export_from_the_avs_eth_0        => eth1g_reg_mosi.rd,
      coe_reg_readdata_export_to_the_avs_eth_0      => eth1g_reg_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_irq_export_to_the_avs_eth_0               => eth1g_reg_interrupt,
      coe_ram_address_export_from_the_avs_eth_0     => eth1g_ram_mosi.address(c_unb1_board_peripherals_mm_reg_default.ram_eth_adr_w-1 DOWNTO 0),
      coe_ram_write_export_from_the_avs_eth_0       => eth1g_ram_mosi.wr,
      coe_ram_writedata_export_from_the_avs_eth_0   => eth1g_ram_mosi.wrdata(c_word_w-1 DOWNTO 0),
      coe_ram_read_export_from_the_avs_eth_0        => eth1g_ram_mosi.rd,
      coe_ram_readdata_export_to_the_avs_eth_0      => eth1g_ram_miso.rddata(c_word_w-1 DOWNTO 0),
  
      -- the_reg_unb_sens
      coe_clk_export_from_the_reg_unb_sens          => OPEN,
      coe_reset_export_from_the_reg_unb_sens        => OPEN,
      coe_address_export_from_the_reg_unb_sens      => reg_unb_sens_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_unb_sens_adr_w-1 DOWNTO 0),
      coe_read_export_from_the_reg_unb_sens         => reg_unb_sens_mosi.rd,
      coe_readdata_export_to_the_reg_unb_sens       => reg_unb_sens_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_write_export_from_the_reg_unb_sens        => reg_unb_sens_mosi.wr,
      coe_writedata_export_from_the_reg_unb_sens    => reg_unb_sens_mosi.wrdata(c_word_w-1 DOWNTO 0),
  
      -- the_reg_dpmm_data
      coe_clk_export_from_the_reg_dpmm_data         => OPEN,
      coe_reset_export_from_the_reg_dpmm_data       => OPEN,
      coe_address_export_from_the_reg_dpmm_data     => reg_dpmm_data_mosi.address(0),
      coe_read_export_from_the_reg_dpmm_data        => reg_dpmm_data_mosi.rd,
      coe_readdata_export_to_the_reg_dpmm_data      => reg_dpmm_data_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_write_export_from_the_reg_dpmm_data       => reg_dpmm_data_mosi.wr,
      coe_writedata_export_from_the_reg_dpmm_data   => reg_dpmm_data_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_dpmm_ctrl
      coe_clk_export_from_the_reg_dpmm_ctrl         => OPEN,
      coe_reset_export_from_the_reg_dpmm_ctrl       => OPEN,
      coe_address_export_from_the_reg_dpmm_ctrl     => reg_dpmm_ctrl_mosi.address(0),
      coe_read_export_from_the_reg_dpmm_ctrl        => reg_dpmm_ctrl_mosi.rd,
      coe_readdata_export_to_the_reg_dpmm_ctrl      => reg_dpmm_ctrl_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_write_export_from_the_reg_dpmm_ctrl       => reg_dpmm_ctrl_mosi.wr,
      coe_writedata_export_from_the_reg_dpmm_ctrl   => reg_dpmm_ctrl_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_mmdp_data
      coe_clk_export_from_the_reg_mmdp_data         => OPEN,
      coe_reset_export_from_the_reg_mmdp_data       => OPEN,
      coe_address_export_from_the_reg_mmdp_data     => reg_mmdp_data_mosi.address(0),
      coe_read_export_from_the_reg_mmdp_data        => reg_mmdp_data_mosi.rd,
      coe_readdata_export_to_the_reg_mmdp_data      => reg_mmdp_data_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_write_export_from_the_reg_mmdp_data       => reg_mmdp_data_mosi.wr,
      coe_writedata_export_from_the_reg_mmdp_data   => reg_mmdp_data_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_mmdp_ctrl
      coe_clk_export_from_the_reg_mmdp_ctrl         => OPEN,
      coe_reset_export_from_the_reg_mmdp_ctrl       => OPEN,
      coe_address_export_from_the_reg_mmdp_ctrl     => reg_mmdp_ctrl_mosi.address(0),
      coe_read_export_from_the_reg_mmdp_ctrl        => reg_mmdp_ctrl_mosi.rd,
      coe_readdata_export_to_the_reg_mmdp_ctrl      => reg_mmdp_ctrl_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_write_export_from_the_reg_mmdp_ctrl       => reg_mmdp_ctrl_mosi.wr,
      coe_writedata_export_from_the_reg_mmdp_ctrl   => reg_mmdp_ctrl_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_epcs
      coe_clk_export_from_the_reg_epcs              => OPEN,
      coe_reset_export_from_the_reg_epcs            => OPEN,
      coe_address_export_from_the_reg_epcs          => reg_epcs_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_epcs_adr_w-1 DOWNTO 0),
      coe_read_export_from_the_reg_epcs             => reg_epcs_mosi.rd,
      coe_readdata_export_to_the_reg_epcs           => reg_epcs_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_write_export_from_the_reg_epcs            => reg_epcs_mosi.wr,
      coe_writedata_export_from_the_reg_epcs        => reg_epcs_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_remu
      coe_clk_export_from_the_reg_remu              => OPEN,
      coe_reset_export_from_the_reg_remu            => OPEN,
      coe_address_export_from_the_reg_remu          => reg_remu_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_remu_adr_w-1 DOWNTO 0),
      coe_read_export_from_the_reg_remu             => reg_remu_mosi.rd,
      coe_readdata_export_to_the_reg_remu           => reg_remu_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_write_export_from_the_reg_remu            => reg_remu_mosi.wr,
      coe_writedata_export_from_the_reg_remu        => reg_remu_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_pio_debug_wave
      out_port_from_the_pio_debug_wave              => OPEN,
  
      -- the_pio_pps
      coe_clk_export_from_the_pio_pps               => OPEN,
      coe_reset_export_from_the_pio_pps             => OPEN,
      coe_address_export_from_the_pio_pps           => reg_ppsh_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_ppsh_adr_w-1),  -- 1 bit address width so must use (0) instead of (0 DOWNTO 0)
      coe_read_export_from_the_pio_pps              => reg_ppsh_mosi.rd,
      coe_readdata_export_to_the_pio_pps            => reg_ppsh_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_write_export_from_the_pio_pps             => reg_ppsh_mosi.wr,
      coe_writedata_export_from_the_pio_pps         => reg_ppsh_mosi.wrdata(c_word_w-1 DOWNTO 0),
                      
      -- the_pio_system_info: actually a avs_common_mm instance
      coe_clk_export_from_the_pio_system_info       => OPEN,
      coe_reset_export_from_the_pio_system_info     => OPEN,
      coe_address_export_from_the_pio_system_info   => reg_unb_system_info_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_unb_system_info_adr_w-1 DOWNTO 0), 
      coe_read_export_from_the_pio_system_info      => reg_unb_system_info_mosi.rd,
      coe_readdata_export_to_the_pio_system_info    => reg_unb_system_info_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_write_export_from_the_pio_system_info     => reg_unb_system_info_mosi.wr,
      coe_writedata_export_from_the_pio_system_info => reg_unb_system_info_mosi.wrdata(c_word_w-1 DOWNTO 0),
  
      -- the_rom_system_info
      coe_clk_export_from_the_rom_system_info       => OPEN,
      coe_reset_export_from_the_rom_system_info     => OPEN,
      coe_address_export_from_the_rom_system_info   => rom_unb_system_info_mosi.address(c_unb1_board_peripherals_mm_reg_default.rom_unb_system_info_adr_w-1 DOWNTO 0), 
      coe_read_export_from_the_rom_system_info      => rom_unb_system_info_mosi.rd,
      coe_readdata_export_to_the_rom_system_info    => rom_unb_system_info_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_write_export_from_the_rom_system_info     => rom_unb_system_info_mosi.wr,
      coe_writedata_export_from_the_rom_system_info => rom_unb_system_info_mosi.wrdata(c_word_w-1 DOWNTO 0),
  
      -- the_pio_wdi: toggled by NIOS II application unb_osy. Connects to WDI via ctrl_unb1_board.
      out_port_from_the_pio_wdi                     => pout_wdi,
  
      -- the_reg_wdi: Manual WDI override; causes FPGA reconfiguration if WDI is enabled (g_use_phy).
      coe_clk_export_from_the_reg_wdi               => OPEN,
      coe_reset_export_from_the_reg_wdi             => OPEN,
      coe_address_export_from_the_reg_wdi           => reg_wdi_mosi.address(0), 
      coe_read_export_from_the_reg_wdi              => reg_wdi_mosi.rd,
      coe_readdata_export_to_the_reg_wdi            => reg_wdi_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_write_export_from_the_reg_wdi             => reg_wdi_mosi.wr,
      coe_writedata_export_from_the_reg_wdi         => reg_wdi_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_tr_10GbE 
      coe_clk_export_from_the_reg_tr_10GbE             => OPEN,
      coe_reset_export_from_the_reg_tr_10GbE           => OPEN,
      coe_address_export_from_the_reg_tr_10GbE         => reg_tr_10GbE_mosi.address(c_reg_tr_10GbE_multi_adr_w-1 DOWNTO 0),
      coe_read_export_from_the_reg_tr_10GbE            => reg_tr_10GbE_mosi.rd,
      coe_readdata_export_to_the_reg_tr_10GbE          => reg_tr_10GbE_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_waitrequest_export_to_the_reg_tr_10GbE       => reg_tr_10GbE_miso.waitrequest,
      coe_write_export_from_the_reg_tr_10GbE           => reg_tr_10GbE_mosi.wr,
      coe_writedata_export_from_the_reg_tr_10GbE       => reg_tr_10GbE_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_tr_xaui 
      coe_clk_export_from_the_reg_tr_xaui              => OPEN,
      coe_reset_export_from_the_reg_tr_xaui            => OPEN,
      coe_address_export_from_the_reg_tr_xaui          => reg_tr_xaui_mosi.address(c_reg_tr_xaui_multi_adr_w-1 DOWNTO 0),
      coe_read_export_from_the_reg_tr_xaui             => reg_tr_xaui_mosi.rd,
      coe_readdata_export_to_the_reg_tr_xaui           => reg_tr_xaui_miso.rddata(c_word_w-1 DOWNTO 0),
      coe_waitrequest_export_to_the_reg_tr_xaui        => reg_tr_xaui_miso.waitrequest,
      coe_write_export_from_the_reg_tr_xaui            => reg_tr_xaui_mosi.wr,
      coe_writedata_export_from_the_reg_tr_xaui        => reg_tr_xaui_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_diag_bg
      reg_diag_bg_address_export                 => reg_diag_bg_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_diag_bg_adr_w-1 DOWNTO 0),
      reg_diag_bg_clk_export                     => OPEN,
      reg_diag_bg_read_export                    => reg_diag_bg_mosi.rd,
      reg_diag_bg_readdata_export                => reg_diag_bg_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_diag_bg_reset_export                   => OPEN,
      reg_diag_bg_write_export                   => reg_diag_bg_mosi.wr,
      reg_diag_bg_writedata_export               => reg_diag_bg_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_ram_diag_bg
      ram_diag_bg_address_export                 => ram_diag_bg_mosi.address(c_ram_diag_bg_addr_w-1 DOWNTO 0),
      ram_diag_bg_clk_export                     => OPEN,
      ram_diag_bg_read_export                    => ram_diag_bg_mosi.rd,
      ram_diag_bg_readdata_export                => ram_diag_bg_miso.rddata(c_word_w-1 DOWNTO 0),
      ram_diag_bg_reset_export                   => OPEN,
      ram_diag_bg_write_export                   => ram_diag_bg_mosi.wr,
      ram_diag_bg_writedata_export               => ram_diag_bg_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_dp_offload_tx
      reg_dp_offload_tx_address_export           => reg_dp_offload_tx_mosi.address(c_reg_dp_offload_tx_multi_adr_w-1 DOWNTO 0),
      reg_dp_offload_tx_clk_export               => OPEN,
      reg_dp_offload_tx_read_export              => reg_dp_offload_tx_mosi.rd,
      reg_dp_offload_tx_readdata_export          => reg_dp_offload_tx_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_dp_offload_tx_reset_export             => OPEN,
      reg_dp_offload_tx_write_export             => reg_dp_offload_tx_mosi.wr,
      reg_dp_offload_tx_writedata_export         => reg_dp_offload_tx_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_dp_offload_tx_hdr_dat
      reg_dp_offload_tx_hdr_dat_address_export   => reg_dp_offload_tx_hdr_dat_mosi.address(c_reg_dp_offload_tx_hdr_dat_multi_adr_w-1 DOWNTO 0),
      reg_dp_offload_tx_hdr_dat_clk_export       => OPEN,
      reg_dp_offload_tx_hdr_dat_read_export      => reg_dp_offload_tx_hdr_dat_mosi.rd,
      reg_dp_offload_tx_hdr_dat_readdata_export  => reg_dp_offload_tx_hdr_dat_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_dp_offload_tx_hdr_dat_reset_export     => OPEN,
      reg_dp_offload_tx_hdr_dat_write_export     => reg_dp_offload_tx_hdr_dat_mosi.wr,
      reg_dp_offload_tx_hdr_dat_writedata_export => reg_dp_offload_tx_hdr_dat_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_dp_offload_tx_hdr_ovr
      reg_dp_offload_tx_hdr_ovr_address_export   => reg_dp_offload_tx_hdr_ovr_mosi.address(c_reg_dp_offload_tx_hdr_ovr_multi_adr_w-1 DOWNTO 0),
      reg_dp_offload_tx_hdr_ovr_clk_export       => OPEN,
      reg_dp_offload_tx_hdr_ovr_read_export      => reg_dp_offload_tx_hdr_ovr_mosi.rd,
      reg_dp_offload_tx_hdr_ovr_readdata_export  => reg_dp_offload_tx_hdr_ovr_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_dp_offload_tx_hdr_ovr_reset_export     => OPEN,
      reg_dp_offload_tx_hdr_ovr_write_export     => reg_dp_offload_tx_hdr_ovr_mosi.wr,
      reg_dp_offload_tx_hdr_ovr_writedata_export => reg_dp_offload_tx_hdr_ovr_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_dp_offload_rx_hdr_dat
      reg_dp_offload_rx_hdr_dat_address_export   => reg_dp_offload_rx_hdr_dat_mosi.address(c_reg_dp_offload_rx_hdr_dat_multi_adr_w-1 DOWNTO 0),
      reg_dp_offload_rx_hdr_dat_clk_export       => OPEN,
      reg_dp_offload_rx_hdr_dat_read_export      => reg_dp_offload_rx_hdr_dat_mosi.rd,
      reg_dp_offload_rx_hdr_dat_readdata_export  => reg_dp_offload_rx_hdr_dat_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_dp_offload_rx_hdr_dat_reset_export     => OPEN,
      reg_dp_offload_rx_hdr_dat_write_export     => reg_dp_offload_rx_hdr_dat_mosi.wr,
      reg_dp_offload_rx_hdr_dat_writedata_export => reg_dp_offload_rx_hdr_dat_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_bsn_monitor
      reg_bsn_monitor_address_export             => reg_bsn_monitor_mosi.address(c_reg_rsp_bsn_monitor_adr_w-1 DOWNTO 0),
      reg_bsn_monitor_clk_export                 => OPEN,
      reg_bsn_monitor_read_export                => reg_bsn_monitor_mosi.rd,
      reg_bsn_monitor_readdata_export            => reg_bsn_monitor_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_bsn_monitor_reset_export               => OPEN,
      reg_bsn_monitor_write_export               => reg_bsn_monitor_mosi.wr,
      reg_bsn_monitor_writedata_export           => reg_bsn_monitor_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_ram_diag_data_buffer
      ram_diag_data_buffer_address_export        => ram_diag_data_buf_mosi.address(c_unb1_board_peripherals_mm_reg_default.ram_diag_db_adr_w-1 DOWNTO 0),
      ram_diag_data_buffer_clk_export            => OPEN,
      ram_diag_data_buffer_read_export           => ram_diag_data_buf_mosi.rd,
      ram_diag_data_buffer_readdata_export       => ram_diag_data_buf_miso.rddata(c_word_w-1 DOWNTO 0),
      ram_diag_data_buffer_reset_export          => OPEN,
      ram_diag_data_buffer_write_export          => ram_diag_data_buf_mosi.wr,
      ram_diag_data_buffer_writedata_export      => ram_diag_data_buf_mosi.wrdata(c_word_w-1 DOWNTO 0),

      -- the_reg_diag_data_buffer
      reg_diag_data_buffer_address_export        => reg_diag_data_buf_mosi.address(c_unb1_board_peripherals_mm_reg_default.reg_diag_db_adr_w-1 DOWNTO 0),
      reg_diag_data_buffer_clk_export            => OPEN,
      reg_diag_data_buffer_read_export           => reg_diag_data_buf_mosi.rd,
      reg_diag_data_buffer_readdata_export       => reg_diag_data_buf_miso.rddata(c_word_w-1 DOWNTO 0),
      reg_diag_data_buffer_reset_export          => OPEN,
      reg_diag_data_buffer_write_export          => reg_diag_data_buf_mosi.wr,
      reg_diag_data_buffer_writedata_export      => reg_diag_data_buf_mosi.wrdata(c_word_w-1 DOWNTO 0)
      );
  END GENERATE;

END str;
