------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE work.unb1_board_pkg.ALL;


ENTITY unb1_board_mesh_io IS
  GENERIC (
    g_bus_w : NATURAL := 3  -- use 4 to include the CMU transceives at port (3) else use 3 to only use the HW transceivers on port (2:0)
  );
  PORT (
    tx_serial_2arr  : IN  t_unb1_board_mesh_sl_2arr := (OTHERS=>(OTHERS=>'0'));
    rx_serial_2arr  : OUT t_unb1_board_mesh_sl_2arr;
    
    -- Serial I/O
    FN_BN_0_TX      : OUT STD_LOGIC_VECTOR(g_bus_w-1 DOWNTO 0);
    FN_BN_0_RX      : IN  STD_LOGIC_VECTOR(g_bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_1_TX      : OUT STD_LOGIC_VECTOR(g_bus_w-1 DOWNTO 0);
    FN_BN_1_RX      : IN  STD_LOGIC_VECTOR(g_bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_2_TX      : OUT STD_LOGIC_VECTOR(g_bus_w-1 DOWNTO 0);
    FN_BN_2_RX      : IN  STD_LOGIC_VECTOR(g_bus_w-1 DOWNTO 0) := (OTHERS=>'0');
    FN_BN_3_TX      : OUT STD_LOGIC_VECTOR(g_bus_w-1 DOWNTO 0);
    FN_BN_3_RX      : IN  STD_LOGIC_VECTOR(g_bus_w-1 DOWNTO 0) := (OTHERS=>'0')
  );
END unb1_board_mesh_io;


ARCHITECTURE str OF unb1_board_mesh_io IS
BEGIN

  -- Map the serial streams to the mesh
  wires : FOR I IN 0 TO g_bus_w-1 GENERATE
    FN_BN_0_TX(I)        <= tx_serial_2arr(0)(I);
    FN_BN_1_TX(I)        <= tx_serial_2arr(1)(I);
    FN_BN_2_TX(I)        <= tx_serial_2arr(2)(I);
    FN_BN_3_TX(I)        <= tx_serial_2arr(3)(I);
  
    rx_serial_2arr(0)(I) <= FN_BN_0_RX(I);
    rx_serial_2arr(1)(I) <= FN_BN_1_RX(I);
    rx_serial_2arr(2)(I) <= FN_BN_2_RX(I);
    rx_serial_2arr(3)(I) <= FN_BN_3_RX(I);
  END GENERATE;

END;












