-------------------------------------------------------------------------------
--
-- Copyright (C) 2012
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

LIBRARY IEEE, common_lib, dp_lib;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.NUMERIC_STD.ALL;
USE common_lib.common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE work.unb1_board_pkg.ALL;

PACKAGE tb_unb1_board_pkg IS

  -- Aggregate types to contain all TR for all nodes on one side of the mesh or backplane interface
  
  -- UniBoard TR mesh IO for 4 to 4 nodes : [node][bus][lane]
  TYPE t_unb1_board_mesh_sosi_3arr IS ARRAY (c_unb1_board_nof_node-1 DOWNTO 0) OF t_unb1_board_mesh_sosi_2arr;
  TYPE t_unb1_board_mesh_siso_3arr IS ARRAY (c_unb1_board_nof_node-1 DOWNTO 0) OF t_unb1_board_mesh_siso_2arr;
  TYPE t_unb1_board_mesh_sl_3arr   IS ARRAY (c_unb1_board_nof_node-1 DOWNTO 0) OF t_unb1_board_mesh_sl_2arr;
  
  -- Subrack with mesh IO for 4 UniBoards, each with TR mesh for 4 to 4 nodes : [unb][node][bus][lane]
  TYPE t_unb1_board_mesh_sosi_4arr IS ARRAY (c_unb1_board_nof_uniboard-1 DOWNTO 0) OF t_unb1_board_mesh_sosi_3arr;
  TYPE t_unb1_board_mesh_siso_4arr IS ARRAY (c_unb1_board_nof_uniboard-1 DOWNTO 0) OF t_unb1_board_mesh_siso_3arr;
  TYPE t_unb1_board_mesh_sl_4arr   IS ARRAY (c_unb1_board_nof_uniboard-1 DOWNTO 0) OF t_unb1_board_mesh_sl_3arr;
  
  -- UniBoard TR backplane IO for 4 BN, each BN with TR to 3 BN each on the 3 other UniBoards [bn][bus][lane]
  TYPE t_unb1_board_back_sosi_3arr IS ARRAY (c_unb1_board_nof_node-1 DOWNTO 0) OF t_unb1_board_back_sosi_2arr;
  TYPE t_unb1_board_back_siso_3arr IS ARRAY (c_unb1_board_nof_node-1 DOWNTO 0) OF t_unb1_board_back_siso_2arr;
  TYPE t_unb1_board_back_sl_3arr   IS ARRAY (c_unb1_board_nof_node-1 DOWNTO 0) OF t_unb1_board_back_sl_2arr;

  -- UniBoard TR backplane IO for 4 UniBoards * 3 buses to other uniboards : [unb][bn][bus][lane]
  TYPE t_unb1_board_back_sosi_4arr IS ARRAY (c_unb1_board_nof_uniboard-1 DOWNTO 0) OF t_unb1_board_back_sosi_3arr;
  TYPE t_unb1_board_back_siso_4arr IS ARRAY (c_unb1_board_nof_uniboard-1 DOWNTO 0) OF t_unb1_board_back_siso_3arr;
  TYPE t_unb1_board_back_sl_4arr   IS ARRAY (c_unb1_board_nof_uniboard-1 DOWNTO 0) OF t_unb1_board_back_sl_3arr;

  TYPE t_unb1_board_id_arr IS ARRAY (NATURAL RANGE <>) OF STD_LOGIC_VECTOR(c_unb1_board_aux.id_w-1 DOWNTO 0);  
END tb_unb1_board_pkg;


PACKAGE BODY tb_unb1_board_pkg IS
END tb_unb1_board_pkg;
