-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------


-- Purpose : Create one gxb_reconfig module for all ALTGX instances.
   

LIBRARY ieee, technology_lib;
USE ieee.std_logic_1164.ALL;
USE technology_lib.technology_pkg.ALL;

ENTITY ip_stratixiv_gxb_reconfig_v111 IS
  GENERIC (
    g_nof_gx        : NATURAL;
    g_fromgxb_bus_w : NATURAL := 17;
    g_togxb_bus_w   : NATURAL := 4
  );
  PORT (
    reconfig_clk     : IN STD_LOGIC;
    reconfig_fromgxb : IN STD_LOGIC_VECTOR(tech_ceil_div(g_nof_gx, 4)*g_fromgxb_bus_w-1 DOWNTO 0);
    busy             : OUT STD_LOGIC;
    reconfig_togxb   : OUT STD_LOGIC_VECTOR(g_togxb_bus_w-1 DOWNTO 0)
  );
END ip_stratixiv_gxb_reconfig_v111;


ARCHITECTURE str OF ip_stratixiv_gxb_reconfig_v111 IS
BEGIN

  gen_gxb_reconfig_4 : IF g_nof_gx <= 4 GENERATE
    u_gxb_reconfig_4 : ENTITY work.ip_stratixiv_gxb_reconfig_v111_4
    PORT MAP (
      reconfig_clk        => reconfig_clk,
      reconfig_fromgxb    => reconfig_fromgxb,
      busy                => busy,
      reconfig_togxb      => reconfig_togxb
    );
  END GENERATE;
        
  gen_gxb_reconfig_16 : IF g_nof_gx > 12 AND g_nof_gx <= 16 GENERATE
    u_gxb_reconfig_16 : ENTITY work.ip_stratixiv_gxb_reconfig_v111_16
    PORT MAP (
      reconfig_clk        => reconfig_clk,
      reconfig_fromgxb    => reconfig_fromgxb,
      busy                => busy,
      reconfig_togxb      => reconfig_togxb
    );
  END GENERATE;
  
END str;

